/*
 * Copyright (c) 2010.  The Apache Software Foundation
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hadoop.hbase.hbql.mapping;

import org.apache.hadoop.hbase.hbql.client.HBqlException;

import java.io.Serializable;
import java.util.List;

public class FamilyMapping implements Serializable {

    private static final long serialVersionUID = 1L;

    private String familyName;
    private List<ColumnDefinition> columnDefinitionList;
    private boolean includeUnmapped;

    public FamilyMapping() {
    }

    public FamilyMapping(final String familyName,
                         final boolean includeUnmapped,
                         final List<ColumnDefinition> columnDefinitionList) {
        this.familyName = familyName;
        this.includeUnmapped = includeUnmapped;
        this.columnDefinitionList = columnDefinitionList;

        if (this.getColumnDefinitionList() != null)
            for (final ColumnDefinition columnDefinition : this.getColumnDefinitionList())
                columnDefinition.setFamilyMapping(this);
    }

    public String getFamilyName() {
        return this.familyName;
    }

    public boolean includeUnmapped() {
        return this.includeUnmapped;
    }

    public List<ColumnDefinition> getColumnDefinitionList() {
        return this.columnDefinitionList;
    }

    public void validate() throws HBqlException {
        if (this.getColumnDefinitionList() != null)
            for (final ColumnDefinition columnDefinition : this.getColumnDefinitionList())
                columnDefinition.getColumnWidth().validate();
    }
}
